# Copyright © 2012-2023 jrnl contributors
# License: https://www.gnu.org/licenses/gpl-3.0.html

from jrnl.messages import Message
from jrnl.messages import MsgStyle
from jrnl.messages import MsgText

Message = Message.Message
MsgStyle = MsgStyle.MsgStyle
MsgText = MsgText.MsgText
